//
// Class to get and store input data
// version 1.2.2002
// (c) 2002 Ingo D. Rullhusen <d01c@uni-bremen.de>
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
//

#include <iostream.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>

#include "input_data.h"


static void option_error( int xi, int argc )
{
   if ( ! ( xi < argc ) )
   {
      cerr << "Error: additional option is missing!" << endl;
      exit(1);
   }
}


InputData::InputData( int argc, char *argv[] )
: max_n(50), ffno(181), theta_inc(0.5*M_PI),
  phi(0.0), rho(0.0), z(0.0),
  eps_r_1(1.0,0.0), eps_r_2(1.0,0.0),
  my_r_1(1.0,0.0), my_r_2(1.0,0.0),
  E_ampl(1.0,0.0),
  ff_file("farfield.dat")
{
   int i;

   for ( i=0; i<argc; i++ )
   {
      if ( strcmp(argv[i],"-n") == 0 )
      {
         option_error(i+1,argc);
         max_n = abs(atoi(argv[i+1]));
      }
      else if ( strcmp(argv[i],"-fn") == 0 )
      {
         option_error(i+1,argc);
         ffno = abs(atoi(argv[i+1]));
      }
      else if ( strcmp(argv[i],"-radius") == 0 )
      {
         option_error(i+1,argc);
         a = fabs(atof(argv[i+1]));
      }
      else if ( strcmp(argv[i],"-frequency") == 0 )
      {
         option_error(i+1,argc);
         freq = fabs(atof(argv[i+1]));
      }
      else if ( strcmp(argv[i],"-theta") == 0 )
      {
         option_error(i+1,argc);
         theta_inc = atof(argv[i+1])*M_PI/180.0;
      }
      else if ( strcmp(argv[i],"-e1") == 0 )
      {
         option_error(i+2,argc);
         eps_r_1 = complex<double>( atof(argv[i+1]), atof(argv[i+2]) );
      }
      else if ( strcmp(argv[i],"-e2") == 0 )
      {
         option_error(i+2,argc);
         eps_r_2 = complex<double>( atof(argv[i+1]), atof(argv[i+2]) );
      }
      else if ( strcmp(argv[i],"-m1") == 0 )
      {
         option_error(i+2,argc);
         my_r_1 = complex<double>( atof(argv[i+1]), atof(argv[i+2]) );
      }
      else if ( strcmp(argv[i],"-m2") == 0 )
      {
         option_error(i+2,argc);
         my_r_2 = complex<double>( atof(argv[i+1]), atof(argv[i+2]) );
      }
      else if ( strcmp(argv[i],"-E") == 0 )
      {
         option_error(i+2,argc);
         E_ampl = complex<double>( atof(argv[i+1]), atof(argv[i+2]) ); 
      }
      else if ( strcmp(argv[i],"-file") == 0 )
      {
         option_error(i+1,argc);
         ff_file = argv[i+1];
      }
      else if ( strcmp(argv[i],"-phi") == 0 )
      {
         option_error(i+1,argc);
         phi = atof(argv[i+1])*M_PI/180.0;
      }
      else if ( strcmp(argv[i],"-rho") == 0 )
      {
         option_error(i+1,argc);
         rho = atof(argv[i+1]);
      }
      else if ( strcmp(argv[i],"-z") == 0 )
      {
         option_error(i+1,argc);
         z = atof(argv[i+1]);
      }
      else if ( strcmp(argv[i],"-h") == 0 )
      {
         cout << "usage: WaitCyl options" << endl;
         cout << " -h               this help" << endl;
         cout << " -n  INTEGER      maximum sum index" << endl;
         cout << " -fn INTEGER      number of far-field samples" << endl;
         cout << " -radius FLOAT    radius of the cylinder (m)" << endl;
         cout << " -frequency FLOAT frequency (Hz)" << endl;
         cout << " -theta FLOAT     incident angle to the negative z axis ()" << endl;
         cout << " -e1 FLOAT FLOAT  relative permitivity inside (Re,Im)" << endl;
         cout << " -e2 FLOAT FLOAT  relative permitivity outside (Re,Im)" << endl;
         cout << " -m1 FLOAT FLOAT  relative permeability inside (Re,Im)" << endl;
         cout << " -m2 FLOAT FLOAT  relative permeability outside (Re,Im)" << endl;
         cout << " -E FLOAT FLOAT   amplitude of incident plane wave (Re,Im)" << endl;
         cout << " -file STRING     file name for far-field output" << endl;
         cout << " -phi FLOAT      phi for inner Ez probe point ()" << endl;
         cout << " -rho FLOAT      rho for inner Ez probe point (m)" << endl;
         cout << " -z   FLOAT      z for inner Ez probe point (m)" << endl;
         cout << endl;
         cout << " most options are required." << endl;
         exit(1);
      }
   }

}


InputData::~InputData() 
{
}


void InputData::out()
{
   cout << " -n " << max_n << endl;
   cout << " -fn " << ffno << endl;
   cout << " -radius " << a << endl;
   cout << " -frequency " << freq << endl;
   cout << " -theta " << theta_inc*180.0/M_PI << endl;
   cout << " -e1 " << eps_r_1.real() << " " << eps_r_1.imag() << endl;
   cout << " -e2 " << eps_r_2.real() << " " << eps_r_2.imag() << endl;
   cout << " -m1 " << my_r_1.real() << " " << my_r_1.imag() << endl;
   cout << " -m2 " << my_r_2.real() << " " << my_r_2.imag() << endl;
   cout << " -E " << E_ampl.real() << " " << E_ampl.imag() << endl;
   cout << " -file " << ff_file << endl;
   cout << " -phi " << phi*180.0/M_PI << endl;
   cout << " -rho " << rho << endl;
   cout << " -z " << z << endl;
}
